#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2015-2015 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require 'singleton'
require_relative 'logging'
require_relative 'configuration'
require 'json'

=begin
   Defines and writes the default mapping file and a default template.
=end
class VCFMapping
	include Singleton
	include Logging
=begin
	See https://en.wikipedia.org/wiki/VCard
=end

	@@VCF4 = %w~BEGIN:VCARD
VERSION:4.0
N:%-name-%;%-first_name-%;%--%;%-title-%
FN:%-full_name-%
ORG:%-organization-%
TITLE:%-position-%
PHOTO;MEDIATYPE=%-image_type-%:%-image_url-%
TEL;TYPE=work,voice;VALUE=uri:tel:%-tel_work-%
TEL;TYPE=home,voice;VALUE=uri:tel:%-tel_home-%
ADR;TYPE=work;LABEL="%-full_addr_work-%":;;%-addr_work-%;%-town_work-%;%-region_work-%;%-post_code_work-%;%-country_work-%
ADR;TYPE=home;LABEL="%-full_addr_home-%:;;%-addr_home-%;%-town_home-%;%-region_home-%;%-post_code_home-%;%-coutry_home-%
EMAIL:%-mail_address-%
REV:%-revision_TZ-%
END:VCARD ~	

# key ex db, value ex VCF
@@DEF_MAPPING = {
	:NAME1 => :name, 
	:VORNAME => :first_name,
	:title => :title,
	:full_name => :full_name,
	:organization => :organization,
	:position => :position,
	:tel_work => :tel_work,
	:tel_home => :tel_home,
	:STRASSE => :addr_work,
	:town_work => :town_work,
	:region_work => :region_work,
	:PLZ => :post_code_work,
	:country_work => :country_work,
	:addr_home => :addr_home,
	:ORT => :town_home,
	:region_home => :region_home,
	:post_code_home => :post_code_home,
	:country_home => :country_home,
	:mail_address => :mail_address,
}

	def initialize
		init_logger();		
		@log.level = Configuration.instance.debug ? Logger::DEBUG : @def_log_level
	end

	def template
		@log.debug('returning default template')
		return JSON.generate(@@VCF4)
	end
	def mapping
		return JSON.generate(@@DEF_MAPPING)
	end
 
end


